"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assembling_machine_renderer_1 = __importDefault(require("../../render/renderers/assembling-machine.renderer"));
const generic_renderer_1 = __importDefault(require("../../render/renderers/generic.renderer"));
const heat_pipe_renderer_1 = __importDefault(require("../../render/renderers/heat-pipe.renderer"));
const pipe_to_ground_renderer_1 = __importDefault(require("../../render/renderers/pipe-to-ground.renderer"));
const pipe_renderer_1 = __importDefault(require("../../render/renderers/pipe.renderer"));
const transport_belt_renderer_1 = __importDefault(require("../../render/renderers/transport-belt.renderer"));
const underground_belt_renderer_1 = __importDefault(require("../../render/renderers/underground-belt.renderer"));
const wall_renderer_1 = __importDefault(require("../../render/renderers/wall.renderer"));
const renderers = {
    "pipe": new pipe_renderer_1.default(),
    "stone-wall": new wall_renderer_1.default(),
    "straight-rail": undefined,
    "curved-rail": undefined,
    "arithmetic-combinator": undefined,
    "decider-combinator": undefined,
    "constant-combinator": undefined,
    "rail-chain-signal": undefined,
    "rail-signal": undefined,
    "beacon": undefined,
    "electric-furnace": undefined,
    "centrifuge": undefined,
    "transport-belt": new transport_belt_renderer_1.default(),
    "fast-transport-belt": new transport_belt_renderer_1.default(),
    "express-transport-belt": new transport_belt_renderer_1.default(),
    "train-stop": undefined,
    "heat-pipe": new heat_pipe_renderer_1.default(),
    "flamethrower-turret": undefined,
    "heat-exchanger": undefined,
    "gate": undefined,
    "burner-inserter": undefined,
    "inserter": undefined,
    "fast-inserter": undefined,
    "long-handed-inserter": undefined,
    "stack-inserter": undefined,
    "filter-inserter": undefined,
    "stack-filter-inserter": undefined,
    "boiler": undefined,
    "steam-engine": undefined,
    "splitter": undefined,
    "fast-splitter": undefined,
    "express-splitter": undefined,
    "underground-belt": new underground_belt_renderer_1.default(),
    "fast-underground-belt": new underground_belt_renderer_1.default(),
    "express-underground-belt": new underground_belt_renderer_1.default(),
    "pipe-to-ground": new pipe_to_ground_renderer_1.default(),
    "assembling-machine": new assembling_machine_renderer_1.default(),
    "assembling-machine-2": new assembling_machine_renderer_1.default(),
    "assembling-machine-3": new assembling_machine_renderer_1.default(),
    "rocket-silo": undefined,
    "gun-turret": undefined,
    "laser-turret": undefined,
    "lab": undefined,
    "power-switch": undefined,
    "small-lamp": undefined,
    "roboport": undefined,
    "steam-turbine": undefined,
    "nuclear-reactor": undefined,
};
const genericRenderer = new generic_renderer_1.default();
class Entity {
    constructor(entity, item, recipe) {
        this.entity = entity;
        this.item = item;
        this.recipe = recipe;
    }
    getTileSize(ent) {
        if (!this.entity.selection_box) {
            return {
                x: 1,
                y: 1
            };
        }
        const min = this.entity.selection_box[0];
        const max = this.entity.selection_box[1];
        if (ent.direction === 2 || ent.direction === 6) {
            return {
                x: Math.ceil(Math.abs(min[1]) + Math.abs(max[1])),
                y: Math.ceil(Math.abs(min[0]) + Math.abs(max[0]))
            };
        }
        return {
            x: Math.ceil(Math.abs(min[0]) + Math.abs(max[0])),
            y: Math.ceil(Math.abs(min[1]) + Math.abs(max[1]))
        };
    }
    getSprite(entity, pass, highRes = false, animationFrame = 0, grid) {
        return __awaiter(this, void 0, void 0, function* () {
            const renderer = entity.name in renderers ? renderers[entity.name] : genericRenderer;
            if (renderer) {
                return renderer.renderPass(entity, pass, highRes, animationFrame, grid);
            }
            // console.log("Failed to render:", entity.name);
            return {};
        });
    }
}
exports.Entity = Entity;
//# sourceMappingURL=entity.js.map