"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const canvas_1 = require("canvas");
const ItemData_1 = require("../ItemData");
const generic_renderer_1 = __importDefault(require("./generic.renderer"));
class TransportBeltRenderer extends generic_renderer_1.default {
    renderPass(entity, pass, highRes = false, animationFrame = 0, grid) {
        return __awaiter(this, void 0, void 0, function* () {
            const ent = ItemData_1.getEntity(entity.name);
            if (!ent) {
                throw new Error("Entity '" + entity.name + "' doesn't exist!");
            }
            if (pass !== "ENTITY") {
                return {};
            }
            let source = ent.entity.belt_vertical;
            if (entity.direction === 2 || entity.direction === 6) {
                source = ent.entity.belt_horizontal;
            }
            const around = this.getAround(entity.direction || 0, entity.position, grid);
            const count = around.reduce((a, b) => a + b, 0);
            const direction = entity.direction || 0;
            let yPos = 0;
            let rotation = 90;
            switch (count) {
                default:
                case 0:
                case 2:
                case 3:
                    if (direction === 0 || direction === 4) {
                        rotation = 0;
                    }
                    break;
                case 1:
                    if (around[0]) {
                        switch (direction) {
                            default:
                            case 0:
                            case 4:
                                rotation = 0;
                                break;
                            case 2:
                                yPos = 9;
                                break;
                            case 6:
                                yPos = 8;
                                rotation = 0;
                                break;
                        }
                    }
                    else if (around[1]) {
                        if (direction === 0) {
                            yPos = 8;
                            rotation = 0;
                        }
                        else if (direction === 4) {
                            yPos = 9;
                        }
                    }
                    else if (around[2]) {
                        switch (direction) {
                            default:
                            case 0:
                            case 4:
                                rotation = 0;
                                break;
                            case 2:
                                yPos = 8;
                                rotation = 0;
                                break;
                            case 6:
                                yPos = 9;
                                break;
                        }
                    }
                    else if (around[3]) {
                        if (direction === 0) {
                            yPos = 9;
                        }
                        else if (direction === 4) {
                            yPos = 8;
                            rotation = 0;
                        }
                    }
                    break;
            }
            return this.loadSprite(source, entity, pass, "", highRes, animationFrame, grid, (canvas, ctx) => {
                // TODO Bend based on surroundings
                const newCanvas = canvas_1.createCanvas(canvas.width, canvas.height);
                const newCtx = newCanvas.getContext("2d");
                if (!newCtx) {
                    throw new Error("Unable to create canvas");
                }
                newCtx.translate(canvas.width / 2, canvas.height / 2);
                newCtx.rotate((((entity.direction || 0) * 45) - rotation) * Math.PI / 180);
                newCtx.drawImage(canvas, (canvas.width / 2) * -1, (canvas.height / 2) * -1);
                return newCanvas;
            }, yPos);
        });
    }
    getKey(entity, pass, highRes, animationFrame, grid) {
        return entity.name + "_" + (entity.direction || 0) + "_" + pass + "_" + this.getAround(entity.direction || 0, entity.position, grid).join("_") + "_" + animationFrame;
    }
    getAround(direction, pos, grid) {
        return [
            pos.x in grid.belts && pos.y - 0.5 in grid.belts[pos.x] && direction !== 0,
            pos.x + 0.5 in grid.belts && pos.y in grid.belts[pos.x + 0.5] && direction !== 2,
            pos.x in grid.belts && pos.y + 0.5 in grid.belts[pos.x] && direction !== 4,
            pos.x - 0.5 in grid.belts && pos.y in grid.belts[pos.x - 0.5] && direction !== 6
        ];
    }
}
exports.default = TransportBeltRenderer;
//# sourceMappingURL=transport-belt.renderer.js.map