"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ItemData_1 = require("../ItemData");
const generic_renderer_1 = __importDefault(require("./generic.renderer"));
class WallRenderer extends generic_renderer_1.default {
    renderPass(entity, pass, highRes = false, animationFrame = 0, grid) {
        return __awaiter(this, void 0, void 0, function* () {
            const ent = ItemData_1.getEntity(entity.name);
            if (!ent) {
                throw new Error(`Entity '${entity.name}' doesn't exist!`);
            }
            const pictures = ent.entity.pictures;
            if (!pictures || !("straight_horizontal" in pictures)) {
                throw new Error(`Unable to render ${entity.name}`);
            }
            const around = this.getAround(entity.position, grid);
            const count = around.reduce((a, b) => a + b, 0);
            let source = pictures.single;
            switch (count) {
                default:
                case 0:
                    break;
                case 1:
                    if (around[1]) {
                        source = pictures.ending_right;
                    }
                    else if (around[2]) {
                        source = pictures.straight_vertical[0];
                    }
                    else {
                        source = pictures.ending_left;
                    }
                    break;
                case 2:
                    if (around[0]) {
                        if (around[1]) {
                            source = pictures.ending_right;
                        }
                        else if (around[2]) {
                            source = pictures.straight_vertical[0];
                        }
                        else if (around[3]) {
                            source = pictures.ending_left;
                        }
                    }
                    else if (around[1]) {
                        if (around[2]) {
                            source = pictures.corner_right_down;
                        }
                        else if (around[3]) {
                            source = pictures.straight_horizontal[0];
                        }
                    }
                    else {
                        source = pictures.corner_left_down;
                    }
                    break;
                case 3:
                    if (!around[0]) {
                        source = pictures.t_up;
                    }
                    else if (!around[1]) {
                        source = pictures.corner_left_down;
                    }
                    else if (!around[2]) {
                        source = pictures.straight_horizontal[0];
                    }
                    else if (!around[3]) {
                        source = pictures.corner_right_down;
                    }
                    break;
                case 4:
                    source = pictures.t_up;
                    break;
            }
            if ("layers" in source) {
                source = source.layers;
            }
            if (Array.isArray(source)) {
                if (pass === "SHADOW") {
                    source = source[1];
                }
                else {
                    source = source[0];
                }
            }
            return this.loadSprite(source, entity, pass, "", highRes, animationFrame, grid);
        });
    }
    getKey(entity, pass, highRes, animationFrame, grid) {
        return entity.name + "_" + (entity.direction || 0) + "_" + pass + "_" + this.getAround(entity.position, grid).join("_") + "_" + animationFrame;
    }
    getAround(pos, grid) {
        return [
            pos.x in grid.walls && pos.y - 0.5 in grid.walls[pos.x] && grid.walls[pos.x][pos.y - 0.5] > 1,
            pos.x + 0.5 in grid.walls && pos.y in grid.walls[pos.x + 0.5] && grid.walls[pos.x + 0.5][pos.y] > 1,
            pos.x in grid.walls && pos.y + 0.5 in grid.walls[pos.x] && grid.walls[pos.x][pos.y + 0.5] > 1,
            pos.x - 0.5 in grid.walls && pos.y in grid.walls[pos.x - 0.5] && grid.walls[pos.x - 0.5][pos.y] > 1
        ];
    }
}
exports.default = WallRenderer;
//# sourceMappingURL=wall.renderer.js.map